#!/sbin/itype
# This is a i file, used by initng parsed by install_service

# This one should be ran first, and have no dependencies.
# Its purpose is to mount /proc /dev and /sys.
service initial/mountvirtfs {
	critical;
	script start = {
		# test, if all necessary directories exist.
		for dir in /proc /sys
		do
			if [ ! -d ${dir} ] # no?
			then
				logger "The dir \"${dir}\" is missing, cant mount it!"
				echo "The dir \"${dir}\" is missing, cant mount it!" >&2
				echo "Please sulogin, remount rw and create them." >&2
				exit 1 # It can't work. Critical!
			fi
		done

		mount -n -t proc proc /proc &
		mount -n -t sysfs sys /sys &

		wait
		exit 0
	};
}

service initial/filldev {
	need = initial/mountvirtfs;
#	use = udev/mountdev;
	critical; # This script can't be fail.
	script start = {
		echo "Mounting devpts at /dev/pts ..."
		/bin/mkdir -p /dev/pts && /bin/mount -n -t devpts -o gid=5,mode=0620 none /dev/pts &

		echo "Mounting devshm at /dev/shm ..."
		/bin/mkdir -p /dev/shm && /bin/mount -n -t tmpfs none /dev/shm &

		# who needs this? only user, who needs this.
		# so i think, we should make this link, if somebody needs this.
		[ ! -e /dev/MAKEDEV -a /sbin/MAKEDEV ] && \
				/bin/ln -fs /sbin/MAKEDEV /dev/MAKEDEV &

		wait
		exit 0
	};
}

service initial/snapshots {
	need = initial/mountvirtfs;
	env EDCONF=/oldroot/snapshots/EDMINI;
	script start = {
		echo "Mounting $EDCONF"
		if [ -d ${EDCONF}/var ]; then
			/bin/mount -n -o bind ${EDCONF}/var /var
			/bin/rm -rf ${EDCONF}/tmp
			/bin/mkdir -p ${EDCONF}/tmp
			/bin/mount -n -o bind ${EDCONF}/tmp /tmp
		fi
	}
	script stop = {
		echo "Cleaning stale PID files"
		find /var/run -name *.pid -exec rm -f {} \;
		echo "Mounting root as RO "
		line="`/oldroot/sbin/unionctl / --list | head -n1`"
		RW_SNAP="`echo $line | awk '{print $1}'`"
		if [ ! -z "$RW_SNAP" ]; then
			kernel_rev=`uname -r | awk -F "." '{print $3}'`
			if [ $kernel_rev -lt 22 ]; then
			    /oldroot/sbin/unionctl / --mode $RW_SNAP ro
			else
			    /bin/mount -t unionfs -o remount,ro none /
			fi
		fi
	}
}

service initial/loglevel {
	# Set the console loglevel to 1 for a cleaner boot
	# the logger should anyhow dump the ring-0 buffer at start to the
	# logs, and that with dmesg can be used to check for problems
	exec start = /bin/dmesg -n 1;
	# Increase the debug level a bit
	exec stop = /bin/dmesg -n 2;
}


service initial/sysctl {
	need = initial/mountvirtfs;
	critical; # This script can't be fail.
	script start = {
		echo 16384 > /proc/sys/fs/inotify/max_user_watches
		exit 0
	};
}

# This is the basic initial
virtual initial {
	critical;
	need = initial/loglevel initial/mountvirtfs initial/sysctl initial/snapshots;
	exec start = export INITNG_PLUGIN_DIR=/lib/initng;
}
