#!/sbin/itype
# This is a i file, used by initng parsed by install_service

virtual udev {
	critical;
	stdall = /dev/null;
	need = udev/filldev udev/udevd udev/set_hotplug;
}

# start the udev daemon
daemon udev/udevd {
	critical;
	need = udev/mountdev initial/mountvirtfs;
	respawn;
	pid_of = udevd;
	forks;
	exec daemon = /sbin/udevd --daemon;
}

service udev/mountdev {
	critical;
	need = initial/mountvirtfs;
	script start = {
		error() {
			echo "${*}" >&2
			exit 1
		}

		[ -e /proc/filesystems ] || error "udev requires a mounted procfs, not started."
		/bin/grep -q '[[:space:]]tmpfs$' /proc/filesystems || error "udev requires tmpfs support, not started."
		[ -d /sys/block ] || error "udev requires a mounted sysfs, not started."

		# THIS service doesn't need this: (I'll remove it 12.1.) (deac)
		# [ -e /proc/sys/kernel/hotplug ] || error "udev requires hotplug support, not started."

		# mount a tmpfs over /dev, if somebody did not already do it
		/bin/grep -Eq "^[^[:space:]]+[[:space:]]+/dev[[:space:]]+tmpfs[[:space:]]+" /proc/mounts && exit 0

		tmpfs_size="10M"

		if ! /bin/mount -n -o size=$tmpfs_size,mode=0755 -t tmpfs udev /dev
		then
			/bin/umount -n /etc/udev
			error "udev in /dev his own filesystem (tmpfs), not started."
		fi

		# Make some default static onces, so we are sure they will exist.
		/bin/mknod -m0666 /dev/null c 1 3
		/bin/mknod -m0666 /dev/zero c 1 5
		/bin/mknod /dev/console c 5 1

		# Send SIGHUP to initng, will reopen /dev/initctl and /dev/initng.
		# we can't assume that initng has pid 1, e.g. when booting from initrd
		#/bin/kill -SIGHUP `pidof initng`
		/bin/kill -SIGHUP `pidof init`
		
		# Why? it's already mounted! (i'll remove it at 12.1., if no reason (deac))
		# [ -d /proc/1 ] || /bin/mount -n /proc
		exit 0
	};
}

# set the hotplug dispatcher
service udev/set_hotplug {
	critical;
	need = udev/udevd initial/mountvirtfs;
	script start = {
		# Make sure hotplug is not missing. The reason, because initng starts sulogin.
		[ -e /proc/sys/kernel/hotplug ] || echo "/proc/sys/kernel/hotplug is missing!"

		unamer=$(uname -r | /bin/sed -e '{ s/^[0-9][0-9]*\.[0-9][0-9]*\.//; s/[\.-].*// }')
		if [ "${unamer}" -ge 15 ]
		then
			# #425: Kernels >= 2.6.15 can directly read udev events via a netlink
			# socket. So there is no need anymore to run udevsend for every event.
			echo ""
		else
			# Setup hotplugging (if possible), if it can't it exits with error.
			echo "/sbin/udevsend"
		fi >/proc/sys/kernel/hotplug
	};
	script stop = {
		# every new hotplug-event can do something wrong.
		echo "" >/proc/sys/kernel/hotplug
	};
}

# fill /dev
service udev/filldev {
	critical;
	need = udev/udevd;
	script start = {
		if [ x"${RC_DEVICE_TARBALL}" = x"/lib/udev/dev_entry.tar.gz" ]
		then
			cd /dev
			/bin/tar -xzf ${RC_DEVICE_TARBALL}
		fi
		# A complete dir to copy
		if [ -d /lib/udev/devices ]
		then
			# Copy over default device tree
			{
				cd /lib/udev/devices &&
				tar c . | tar x -C /dev/
			} &
		fi

		# No-one else creates these; we need to do it manually
		/bin/ln -snf /proc/self/fd /dev/fd
		/bin/ln -snf fd/0 /dev/stdin
		/bin/ln -snf fd/1 /dev/stdout
		/bin/ln -snf fd/2 /dev/stderr
		/bin/ln -snf /proc/kcore /dev/core
		
		/bin/mkdir -p /dev/.udev/db/ /dev/.udev/queue/
                # Place a flag to tell raidmanager to ignore udev requests
                # during init
                touch /dev/.udev/starting

		/sbin/udevadm trigger
		/sbin/udevadm settle
                
                # We can now remove the flag
                rm /dev/.udev/starting
		
		chmod 0666 /dev/null
	};
	script stop = {
		if [ "${RC_DEVICE_TARBALL}" = x"/lib/udev/dev_entry.tar.gz" ]
		then
			cd /dev
			/bin/tar -czf ${RC_DEVICE_TARBALL}
		fi
		exit 0
	};
}
